#!/bin/sh
#
# This script creates an IAP to be used in the specified network.
#
# Run this script after flashing the device.  Specify the network to be configured.
#
# History:
#
# 21.Feb.2008   0.1 - This version has been tested with Camp build wk8-3.
# 22.Feb.2008   0.2 - [CAMP BUILD 0.2008.08-6]
#                     Added some more options and fixed SL2 network
# 28.Feb.2008   0.3 - Fixed some bugs related to idle mode in linuxrc
# 06.Mar.2008   0.4 - Fixed SL2-SAMS settings
# 01.Apr.2008   0.5 - Added Alcatel settings
# 09.Apr.2008   0.6 - Added DM Operator to the IAP settings
# 14.Apr.2008   0.7 - Added ND-MOTO settings
# 30.Apr.2008   0.8 - Don't use idle mode by default
# 06.May.2008   0.9 - Use preferred NAP ID in GConf settings
# 07.May.2008  0.10 - Added Ruoholahti settings
# 08.May.2008  0.11 - NAP ID is saved in DM instead of GConf
# 22.May.2008  0.12 - TD-MOTO now uses real certificates and xohm.com as realm
# 27.May.2008  0.13 - Added PELTOLA-NSN settings
# 02.Jun.2008  0.14 - Added TD-FIELD settings
# 25.Jun.2008  0.16 - Added CLWR. 
# 12.Aug.2008  0.17 - Merged changes for Clearwire
# 20.Aug.2008  0.18 - Added support to change the NAP ID
# 30.Aug.2008  0.19 - Don't reload the driver when disabling idle.

# We use this stuff as default when setting the frequency
BS_ID=010203
FFT_SIZE=1024
BANDWIDTH=10000
PREAMBLE="[255,255,255,255,255,255,255,255,255,255,255,255,255,255,255]"

print_usage () {
    echo "Usage: $PROG <TD-MOTO|ND-SAMS|SL2-SAMS|ALCATEL|RUOHO|PELTOLA-NSN|CLWR> [-h] [-l] [-i] [-f <frequency>] [-u <username> -p <password>] [-I <ip_address> -N <ip_netmask>]"
    echo "       $PROG -h"
    echo
    echo "       TD-MOTO for TD Lab with Motorola BS"
    echo "       ND-SAMS for ND Lab with Samsung BS"
    echo "       ND-MOTO for ND Lab with Motorola BS"
    echo "       SL2-SAMS for Soft Launch 2 Network with Samsung BS"
    echo "       ALCATEL for Alcatel BS"
    echo "       RUOHO for Alvarion BS in Ruoholahti"
    echo "       PELTOLA-NSN for NSN BS in Peltola"
    echo "       TD-FIELD for TD field test network"
    echo "       CLWR for Clearwire network"
    echo
    echo "       -l use lowercase for the MAC address part of the NAI"
    echo "       -u username -- Must be set for TTLS networks"
    echo "       -p password -- Must be set for TTLS networks"
    echo "       -n nai -- force NAI to be used"
    echo "       -I IP address -- Must be set for static IP"
    echo "       -N IP netmask -- Must be set for static IP"
    echo "       -f frequency -- define a frequency to be used (overrides net-specific frequency)"
    echo "       -i use idle mode (default is not idle mode)"
    echo "       -A nap_id -- force NAP_ID to be used"
    echo "       -h print this usage information"
    echo
}


get_mac () {
    MAC=$(ifconfig wimax0|grep HWaddr|sed 's/^.*HWaddr \([0-9A-F][0-9A-F]\):\([0-9A-F][0-9A-F]\):\([0-9A-F][0-9A-F]\):\([0-9A-F][0-9A-F]\):\([0-9A-F][0-9A-F]\):\([0-9A-F][0-9A-F]\).*/\1\2\3\4\5\6/')

    if [ "${LOWERCASE}" = "true" ]; then
	MAC=$(echo "${MAC}" | sed 'y/ABCDEF/abcdef/')
    fi
}

disable_idle () {
    if [ ! "$(grep 'DISABLING PC2400M IDLE' /mnt/initfs/linuxrc)" != "" ]; then
	echo "Disabling IDLE mode in linuxrc startup script..."

	mv /mnt/initfs/linuxrc /mnt/initfs/linuxrc.orig
	sed 's/pc2400m.ko/pc2400m.ko ; echo "DISABLING PC2400M IDLE MODE" ; iwtool config -I 0/' /mnt/initfs/linuxrc.orig > /mnt/initfs/linuxrc
	chmod a+x /mnt/initfs/linuxrc
    fi
}

PROG=$(basename $0)

if [ $# -lt 1 ]; then
    print_usage
    exit 1
fi

if [ "$1" = "-h" ]; then
    print_usage
    exit 0
fi

NETWORK=$1
shift

case ${NETWORK} in
    TD-MOTO)
	IAP=WIMAX000004
	OPERATOR=Xohm
	REALM=xohm.com
	NAP_ID=000004
	NSP_ID=000004
	CERT_DIR=sprint
	DEV_CERTS=0
        # EAP type: 13 EAP TLS
	EAP_TYPE=13
	SERVICE_ID=000004
	NETWORK_TYPE=WIMAX
	SERVICE_TYPE=WIMAX
	IP_TYPE=auto
	SECURITY=WPA_EAP
	CERT_CHECK=1
	OPERATOR_IN_DM=xohm
	;;

    ND-SAMS)
	IAP=WIMAX000004
	OPERATOR=Xohm
	REALM=sprintpcs.com
	NAP_ID=000004
	NSP_ID=000004
	CERT_DIR=sprint
	DEV_CERTS=0
	DECORATION="{am=2}"
	# EAP type: 13 EAP TLS
	EAP_TYPE=13
	SERVICE_ID=000004
	NETWORK_TYPE=WIMAX
	SERVICE_TYPE=WIMAX
	IP_TYPE=auto
	SECURITY=WPA_EAP
	CERT_CHECK=1
	OPERATOR_IN_DM=xohm
	;;

    ND-MOTO)
	IAP=WIMAX000004
	OPERATOR=Xohm
	REALM=xohm.com
	NAP_ID=000007
	NSP_ID=000004
	CERT_DIR=sprint
	DEV_CERTS=1
	DECORATION="{am=2}"
        # EAP type: 13 EAP TLS
	EAP_TYPE=13
	SERVICE_ID=000004
	NETWORK_TYPE=WIMAX
	SERVICE_TYPE=WIMAX
	IP_TYPE=auto
	SECURITY=WPA_EAP
	CERT_CHECK=1
	OPERATOR_IN_DM=xohm
	;;

    SL2-SAMS)
        IAP=WIMAX000004
        OPERATOR=Xohm
        REALM=xohm.com
        NAP_ID=000004
        NSP_ID=000004
        CERT_DIR=sprint
        DEV_CERTS=0
        DECORATION="{am=2}"
        # EAP type: 13 EAP TLS
        EAP_TYPE=13
        SERVICE_ID=000004
        NETWORK_TYPE=WIMAX
        SERVICE_TYPE=WIMAX
        IP_TYPE=auto
        SECURITY=WPA_EAP
	CERT_CHECK=1
	OPERATOR_IN_DM=xohm
        ;; 

    ALCATEL)
        IAP=WIMAX000004
        OPERATOR=Xohm
        NAI=anonymous@tic.com
        NAP_ID=000310
        NSP_ID=000004
        CERT_DIR=sprint
        DEV_CERTS=0
        # EAP type: 21 EAP TTLS
        EAP_TYPE=21
	TUNNELED_EAP_TYPE=99
        SERVICE_ID=000004
        NETWORK_TYPE=WIMAX
        SERVICE_TYPE=WIMAX
        IP_TYPE=auto
        SECURITY=WPA_EAP
	CERT_CHECK=0
	OPERATOR_IN_DM=xohm
	;;

    RUOHO)
        IAP=WIMAX000004
	OPERATOR=Xohm
        SECURITY=NONE
        NAP_ID=000004
        NSP_ID=000004
        SERVICE_ID=000004
        NETWORK_TYPE=WIMAX
        SERVICE_TYPE=WIMAX
        IP_TYPE=static
	DEV_CERTS=0
	CERT_CHECK=0
	OPERATOR_IN_DM=xohm
	if [ "${FREQUENCY}" = "" ]; then
	    FREQUENCY=2600000
	fi
	;;

    PELTOLA-NSN)
        IAP=WIMAX000004
        OPERATOR=Xohm
        NAP_ID=0A2230
        NSP_ID=000004
        DEV_CERTS=0
        # EAP type: 21 EAP TTLS
        EAP_TYPE=21
	TUNNELED_EAP_TYPE=99
        SERVICE_ID=000004
        NETWORK_TYPE=WIMAX
        SERVICE_TYPE=WIMAX
        IP_TYPE=auto
        SECURITY=WPA_EAP
	CERT_CHECK=0
	OPERATOR_IN_DM=xohm
	;;

    TD-FIELD)
	IAP=WIMAX000004
	OPERATOR=Xohm
	REALM=xohm.com
	NAP_ID=000007
	NSP_ID=000004
	CERT_DIR=sprint
	DEV_CERTS=0
        # EAP type: 13 EAP TLS
	EAP_TYPE=13
	SERVICE_ID=000004
	NETWORK_TYPE=WIMAX
	SERVICE_TYPE=WIMAX
	IP_TYPE=auto
	SECURITY=WPA_EAP
	CERT_CHECK=1
	OPERATOR_IN_DM=xohm
	;;

    CLWR)
        IAP=WIMAX000002
        OPERATOR=Clearwire
        REALM=clearwire-wmx.net
        NAP_ID=000002
        NSP_ID=000002
        CERT_DIR=clearwire
        DEV_CERTS=0
        # No DECORATION
        # EAP type: 13 EAP TLS
        EAP_TYPE=13
        SERVICE_ID=000002
        NETWORK_TYPE=WIMAX
        SERVICE_TYPE=WIMAX
        IP_TYPE=auto
        SECURITY=WPA_EAP
	CERT_CHECK=1
	OPERATOR_IN_DM=clearwire
        ;; 

    *)  echo "Invalid network! $NETWORK"
	print_usage
	exit 1
	;;

esac

# TODO: Use getopt instead
while [ $# -ge 1 ]; do
    case $1 in
	-l) 
	    LOWERCASE="true"
	    ;;
	-u)
	    if [ $# -lt 2 ]; then
		echo "Missing username"
		print_usage
		exit 1
	    fi
	    USERNAME=$2
	    shift
	    ;;
	-p)
	    if [ $# -lt 2 ]; then
		echo "Missing password"
		print_usage
		exit 1
	    fi
	    PASSWORD=$2
	    shift
	    ;;
	-n)
	    if [ $# -lt 2 ]; then
		echo "Missing NAI string"
		print_usage
		exit 1
	    fi
	    NAI=$2
	    shift
	    ;;
	-I)
	    if [ $# -lt 2 ]; then
		echo "Missing IP address"
		print_usage
		exit 1
	    fi
	    IP_ADDRESS=$2
	    shift
	    ;;
	-N)
	    if [ $# -lt 2 ]; then
		echo "Missing IP netmask"
		print_usage
		exit 1
	    fi
	    IP_NETMASK=$2
	    shift
	    ;;
	-f)
	    if [ $# -lt 2 ]; then
		echo "Missing frequency"
		print_usage
		exit 1
	    fi
	    FREQUENCY=$2
	    shift
	    ;;
	-i) 
	    USE_IDLE="true"
	    ;;
	-A)
	    if [ $# -lt 2 ]; then
		echo "Missing NAP ID string"
		print_usage
		exit 1
	    fi
	    NAP_ID="$2"
	    shift
	    ;;
	-h)
	    print_usage
	    exit 0
	    ;;
	*)
	    echo "Invalid argument $1"
	    print_usage
	    exit 1
	    ;;
    esac
    shift
done

if [ "${SECURITY}" != "NONE" -a "${NAI}" = "" ]; then
    get_mac
    NAI="${DECORATION}${MAC}@${REALM}"
fi

if [ ${EAP_TYPE} -eq 21 ]; then
    if [ "${USERNAME}" = "" -o "${PASSWORD}" = "" ]; then
	echo "Username and password are required for TTLS networks"
	print_usage
	exit 1
    fi
    gconftool -s /system/osso/connectivity/IAP/${IAP}/EAP_MSCHAPV2_password_prompt -t int 0
    gconftool -s /system/osso/connectivity/IAP/${IAP}/PEAP_tunneled_eap_type -t int ${TUNNELED_EAP_TYPE}
    gconftool -s /system/osso/connectivity/IAP/${IAP}/EAP_MSCHAPV2_username -t string ${USERNAME}
    gconftool -s /system/osso/connectivity/IAP/${IAP}/EAP_MSCHAPV2_password -t string ${PASSWORD}
fi

if [ ${CERT_CHECK} -eq 0 ]; then
    gconftool -s /system/osso/connectivity/IAP/${IAP}/EAP_disable_certificate_check -t boolean true
else
    gconftool -s /system/osso/connectivity/IAP/${IAP}/EAP_disable_certificate_check -t boolean false
fi

echo "Configuring $IAP to be used on the $1 network..."
if [ "${SECURITY}" != "NONE" ]; then
    echo "NAI = ${NAI}"
else
    echo "Not using security."
    EAP_TYPE=0
fi

if [ "${IP_TYPE}" = "static" ]; then
    if [ "${IP_ADDRESS}" = "" -o "${IP_NETMASK}" = "" ]; then
	echo "IP address and netmask are required for static IP addresses."
	print_usage
	exit 1
    fi
    gconftool -s /system/osso/connectivity/IAP/${IAP}/ipv4_address -t string ${IP_ADDRESS}
    gconftool -s /system/osso/connectivity/IAP/${IAP}/ipv4_netmask -t string ${IP_NETMASK}
fi

if [ "${FREQUENCY}" != "" ]; then
    gconftool -s /system/osso/connectivity/IAP/${IAP}/MRUBS/${BS_ID}/nap_id -t int ${NAP_ID}
    gconftool -s /system/osso/connectivity/IAP/${IAP}/MRUBS/${BS_ID}/fft_size -t int ${FFT_SIZE}
    gconftool -s /system/osso/connectivity/IAP/${IAP}/MRUBS/${BS_ID}/carrier_freq -t int ${FREQUENCY}
    gconftool -s /system/osso/connectivity/IAP/${IAP}/MRUBS/${BS_ID}/bandwidth -t int ${BANDWIDTH}
    gconftool -s /system/osso/connectivity/IAP/${IAP}/MRUBS/${BS_ID}/preamble -t list --list-type=int "${PREAMBLE}"
fi    

gconftool -s /system/osso/connectivity/IAP/${IAP}/service_type -t string ${SERVICE_TYPE}
gconftool -s /system/osso/connectivity/IAP/${IAP}/ipv4_type -t string ${IP_TYPE}
gconftool -s /system/osso/connectivity/IAP/${IAP}/nsp_id -t string ${NSP_ID}
gconftool -s /system/osso/connectivity/IAP/${IAP}/type -t string ${NETWORK_TYPE}
gconftool -s /system/osso/connectivity/IAP/${IAP}/wlan_security -t string ${SECURITY}
gconftool -s /system/osso/connectivity/IAP/${IAP}/name -t string ${OPERATOR}
gconftool -s /system/osso/connectivity/IAP/${IAP}/EAP_default_type -t int ${EAP_TYPE}
gconftool -s /system/osso/connectivity/IAP/${IAP}/temporary -t boolean false
gconftool -s /system/osso/connectivity/IAP/${IAP}/service_id -t string ${SERVICE_ID}
gconftool -s /system/osso/connectivity/IAP/${IAP}/nai -t string "${NAI}"
gconftool -s /system/osso/connectivity/IAP/${IAP}/WiMAXOperator -t string "${OPERATOR_IN_DM}"

gconftool -s /system/osso/connectivity/srv_provider/WIMAX/promotion/nap_id_list -t list --list-type=string [${NAP_ID}]

if [ "${SECURITY}" != "NONE" ]; then
    echo "Copying CA certificates from /usr/share/wimax-extras/certs/${CERT_DIR}..."
    mkdir /usr/share/ca-certificates 2> /dev/null
    mkdir /usr/share/ca-certificates/wimax 2> /dev/null
    cp /usr/share/wimax-extras/certs/${CERT_DIR}/cacert.der /usr/share/ca-certificates/wimax/
    cp /usr/share/wimax-extras/certs/${CERT_DIR}/cacert.pem /usr/share/ca-certificates/wimax/
fi

if [ "${DEV_CERTS}" -eq 1 ]; then
    echo "Copying Device certificates from  /usr/share/wimax-extras/certs/${CERT_DIR}..."
    cp /usr/share/wimax-extras/certs/${CERT_DIR}/client2-cert.der /usr/share/certs/devicecert.der
    cp /usr/share/wimax-extras/certs/${CERT_DIR}/client2-cert.pem /usr/share/certs/devicecert.pem
    cp /usr/share/wimax-extras/certs/${CERT_DIR}/client2-key.der /usr/share/certs/devicekey.der 
    cp /usr/share/wimax-extras/certs/${CERT_DIR}/client2-key.pem /usr/share/certs/devicekey.pem 
else
    echo "Removing device certificates files from /usr/share/certs..."
    rm -f /usr/share/certs/devicecert.der
    rm -f /usr/share/certs/devicecert.pem
    rm -f /usr/share/certs/devicekey.der 
    rm -f /usr/share/certs/devicekey.pem 
fi

mount /mnt/initfs -o remount,rw
if [ "$USE_IDLE" = "true" ]; then
    if [ -r "/mnt/initfs/linuxrc.orig" ]; then
	echo "Restoring original linuxrc..."
	mv -f /mnt/initfs/linuxrc.orig /mnt/initfs/linuxrc
    fi
else
    disable_idle
    iwtool config -I 0
fi
mount /mnt/initfs -o remount,ro

NAP_ID_DECIMAL=$(printf "%d" 0x${NAP_ID})

echo "Modifying parameters in DM..."
gconftool -s /system/osso/connectivity/dm/WiMAXSupp/Operator/${OPERATOR_IN_DM}/NetworkParameters/CAPL/Entries/0/NAP-ID -t list --list-type=string [Format:int,Name:NAP-ID,Type:text/plain,Value:${NAP_ID_DECIMAL}]
gconftool -s /system/osso/connectivity/dm/WiMAXSupp/Operator/${OPERATOR_IN_DM}/SubscriptionParameters/Primary/Activated -t list --list-type=string [Format:bool,Name:Activated,Type:text/plain,Value:TRUE]

echo "Updating dmsessiond cache..."
dbus-send --print-reply --system --dest=com.nokia.dm /com/nokia/dm com.nokia.dm.invalidate_cache

echo "Done."
